@@ -1,20 +1,23 @@
-//#include "stdafx.h"
-#include "zlib/zlib.h"
 #include "bzip2/bzlib.h"
 #include "UnAlz.h"
 
+#ifdef _WIN32
+#	include "zlib/zlib.h"
+#else
+#	include <zlib.h>
+#endif
+
+
 // utime Լ ó
 #if defined(_WIN32) || defined(__CYGWIN__)
 #	include <time.h>
 #	include <sys/utime.h>
 #endif
-
 #ifdef __GNUC__
 #	include <time.h>
 #	include <utime.h>
 #endif
 
-
 // mkdir
 #ifdef _WIN32				
 #	include <direct.h>		
@@ -26,61 +29,48 @@
 #	include <iconv.h>
 #endif
 
-#ifdef __linux__			// __BYTE_ORDER  
-#	include <errno.h>		// iconv.h  ʿ 
+#if defined(__linux__) || defined(__GLIBC__) || defined(__GNU__)
+#	include <errno.h>
 #endif
 
 #if defined(__NetBSD__)
-#	include <sys/param.h>		// __NetBSD_Version__
+#	include <sys/param.h>	// __NetBSD_Version__
 #	include <errno.h>		// iconv.h  ʿ 
 #endif
 
-#define swapint64(Data) (INT64) ( (((Data)&0x00000000000000FFLL) << 56) | (((Data)&0x000000000000FF00LL) << 40) | (((Data)&0x0000000000FF0000LL) << 24) | (((Data)&0x00000000FF000000LL) << 8)  | (((Data)&0x000000FF00000000LL) >> 8)  | (((Data)&0x0000FF0000000000LL) >> 24) | (((Data)&0x00FF000000000000LL) >> 40) | (((Data)&0xFF00000000000000LL) >> 56) )
+#ifdef _WIN32				// string conversion
+#	include <atlbase.h>
+#	include <atlconv.h>
+#endif
+
+#ifdef _WIN32				// safe string ó
+#	include <strsafe.h>
+#endif
+
+
+// ENDIAN ó
+#ifdef _WIN32	// (L)
+#	define swapint64(a)	(UINT64) ( (((a)&0x00000000000000FFL) << 56) | (((a)&0x000000000000FF00L) << 40) | (((a)&0x0000000000FF0000L) << 24) | (((a)&0x00000000FF000000L) << 8)  | (((a)&0x000000FF00000000L) >> 8)  | (((a)&0x0000FF0000000000L) >> 24) | (((a)&0x00FF000000000000L) >> 40) | (((a)&0xFF00000000000000L) >> 56) )
+#else			// (LL)
+#	define swapint64(a)	(UINT64) ( (((a)&0x00000000000000FFLL) << 56) | (((a)&0x000000000000FF00LL) << 40) | (((a)&0x0000000000FF0000LL) << 24) | (((a)&0x00000000FF000000LL) << 8)  | (((a)&0x000000FF00000000LL) >> 8)  | (((a)&0x0000FF0000000000LL) >> 24) | (((a)&0x00FF000000000000LL) >> 40) | (((a)&0xFF00000000000000LL) >> 56) )
+#endif
 #define swapint32(a)    ((((a)&0xff)<<24)+(((a>>8)&0xff)<<16)+(((a>>16)&0xff)<<8)+(((a>>24)&0xff)))
 #define swapint16(a)    (((a)&0xff)<<8)+(((a>>8)&0xff))
 
-////////////////////////////////////////////////////////////////////////////
-//// byte-order : little to host                                        ////
-////////////////////////////////////////////////////////////////////////////
+typedef UINT16	(*_unalz_le16toh)(UINT16 a);
+typedef UINT32	(*_unalz_le32toh)(UINT32 a);
+typedef UINT64	(*_unalz_le64toh)(UINT64 a);
 
-#if defined(_WIN32) || defined(__CYGWIN__)		// little to little
-	inline UINT16	unalz_le16toh(UINT16 a){return a;}
-	inline UINT32	unalz_le32toh(UINT32 a){return a;}
-	inline UINT64	unalz_le64toh(UINT64 a){return a;}
-#endif
-
-#if defined(__FreeBSD__) || defined(__NetBSD__)
-#	include <sys/endian.h>	
-	inline UINT16	unalz_le16toh(UINT16 a){return le16toh(a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return le32toh(a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return le64toh(a);}
-#endif
-
-#ifdef __APPLE__
-#	include <machine/byte_order.h>
-	inline UINT16   unalz_le16toh(UINT16 a){return NXSwapShort(a);} 
-	inline UINT32   unalz_le32toh(UINT32 a){return NXSwapLong(a);} 
-	inline UINT64   unalz_le64toh(UINT64 a){return NXSwapLongLong(a);} 
-#endif
-
-
-#ifdef __linux__	
-#	include <endian.h>
-#	if __BYTE_ORDER == __BIG_ENDIAN
-	inline UINT16	unalz_le16toh(UINT16 a){return swapint16(a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return swapint32(a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return swapint64(a);}
-#	else		// __LITTLE_ENDIAN
-	inline UINT16	unalz_le16toh(UINT16 a){return (a);}
-	inline UINT32	unalz_le32toh(UINT32 a){return (a);}
-	inline UINT64	unalz_le64toh(UINT64 a){return (a);}
-#	endif
-//#	include <asm/byteorder.h>
-//	inline UINT16	unalz_le16toh(UINT16 a){return le16_to_cpu(a);}
-//	inline UINT32	unalz_le32toh(UINT32 a){return le32_to_cpu(a);}
-//	inline UINT64	unalz_le64toh(UINT64 a){return le64_to_cpu(a);}
-#endif
+static _unalz_le16toh unalz_le16toh=NULL;
+static _unalz_le32toh unalz_le32toh=NULL;
+static _unalz_le64toh unalz_le64toh=NULL;
 
+static UINT16	le16tole(UINT16 a){return a;}
+static UINT32	le32tole(UINT32 a){return a;}
+static UINT64	le64tole(UINT64 a){return a;}
+static UINT16	le16tobe(UINT16 a){return swapint16(a);}
+static UINT32	le32tobe(UINT32 a){return swapint32(a);}
+static UINT64	le64tobe(UINT64 a){return swapint64(a);}
 
 
 #ifndef MAX_PATH
@@ -108,6 +98,25 @@ static time_t dosTime2TimeT(UINT32 dostime)   // from INFO-ZIP src
 	return mktime(&t);
 }
 
+static BOOL IsBigEndian(void) 
+{
+   union {
+        short a;
+        char  b[2];
+   } endian;
+   
+   endian.a = 0x0102;
+   if(endian.b[0] == 0x02) return FALSE;
+   return TRUE;
+}
+
+
+#ifdef _WIN32
+#	define safe_sprintf	StringCbPrintfA
+#else
+#	define safe_sprintf	snprintf
+#endif
+
 
 // error string table <- CUnAlz::ERR  
 static const char* errorstrtable[]=
@@ -148,7 +157,6 @@ static const char* errorstrtable[]=
 };
 
 
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 ///          ctor
 /// @date    2004-03-06  11:19:49
@@ -157,7 +165,7 @@ CUnAlz::CUnAlz()
 {
 	memset(m_files, 0, sizeof(m_files));
 	m_nErr = ERR_NOERR;
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 	m_pFuncCallBack = NULL;
 	m_pCallbackParam = NULL;
 	m_bHalt = FALSE;
@@ -173,13 +181,30 @@ CUnAlz::CUnAlz()
 #ifdef _UNALZ_ICONV
 
 #ifdef _UNALZ_UTF8
-	strcpy(m_szToCodepage, "UTF-8") ;		// ⺻ utf-8
+	safe_strcpy(m_szToCodepage, "UTF-8",UNALZ_LEN_CODEPAGE) ;		// ⺻ utf-8
 #else 
-	strcpy(m_szToCodepage, "CP949") ;		// ⺻ CP949
+	safe_strcpy(m_szToCodepage, "CP949",UNALZ_LEN_CODEPAGE) ;		// ⺻ CP949
 #endif // _UNALZ_UTF8
 
-	strcpy(m_szFromCodepage, "CP949");		// alz  949  
+	safe_strcpy(m_szFromCodepage, "CP949",UNALZ_LEN_CODEPAGE);		// alz  949  
 #endif // _UNALZ_ICONV
+
+	// check endian
+	if(unalz_le16toh==NULL)
+	{
+		if(IsBigEndian())
+		{
+			unalz_le16toh = le16tobe;
+			unalz_le32toh = le32tobe;
+			unalz_le64toh = le64tobe;
+		}
+		else
+		{
+			unalz_le16toh = le16tole;
+			unalz_le32toh = le32tole;
+			unalz_le64toh = le64tole;
+		}
+	}
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -209,8 +234,6 @@ void CUnAlz::SetCallback(_UnAlzCallback* pFunc, void* param)
 /// @return  
 /// @date    2004-03-06  11:03:59
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-#include <atlbase.h>
-#include <atlconv.h>
 BOOL CUnAlz::Open(LPCWSTR szPathName)
 {
 	USES_CONVERSION;
@@ -314,7 +337,7 @@ void CUnAlz::Close()
 		i->Clear();
 	}
 
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 }
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -327,7 +350,6 @@ CUnAlz::SIGNATURE CUnAlz::ReadSignature()
 	UINT32	dwSig;
 	if(FRead(&dwSig, sizeof(dwSig))==FALSE)
 	{
-//int pos = ftell(m_fp);
 		if(FEof())
 			return SIG_EOF;
 		m_nErr = ERR_CANT_READ_SIG;
@@ -351,7 +373,6 @@ BOOL CUnAlz::ReadAlzFileHeader()
 		m_nErr = ERR_CANT_READ_FILE;
 		return FALSE;
 	}
-
 	return TRUE;
 }
 
@@ -362,7 +383,7 @@ BOOL CUnAlz::ReadAlzFileHeader()
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 BOOL CUnAlz::ReadLocalFileheader()
 {
-	SLocalFileHeader zipHeader;
+	SAlzLocalFileHeader zipHeader;
 	int ret;
 
 	ret = FRead(&(zipHeader.head), sizeof(zipHeader.head));
@@ -494,7 +515,7 @@ BOOL CUnAlz::ReadLocalFileheader()
 	*/
 
 	if(IsEncryptedFile(zipHeader.head.fileDescriptor)) 
-		FRead(zipHeader.encChk, ENCR_HEADER_LEN);  // xf86
+		FRead(zipHeader.encChk, ALZ_ENCR_HEADER_LEN);  // xf86
 
 	// SKIP FILE DATA
 	zipHeader.dwFileDataPos = FTell();						// data  ġ ϱ..
@@ -574,7 +595,6 @@ BOOL CUnAlz::ReadCentralDirectoryStructure()
 		FRead(header.fileComment, 1, header.head.fileCommentLength, m_fp);
 		header.fileComment[header.head.fileCommentLength] = NULL;
 	}
-
 	*/
 
 	return TRUE;
@@ -627,7 +647,7 @@ BOOL CUnAlz::SetCurrentFile(const char* szFileName)
 		}
 	}
 
-	m_posCur = (FileList::iterator)NULL;
+	m_posCur = m_fileList.end();//(FileList::iterator)NULL;
 
 	return FALSE;
 }
@@ -668,8 +688,7 @@ BOOL CUnAlz::ExtractCurrentFileToBuf(BYTE* pDestBuf, int nBufSize)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 BOOL CUnAlz::ExtractCurrentFile(const char* szDestPathName, const char* szDestFileName)
 {
-	if(m_posCur==(FileList::iterator)NULL) {ASSERT(0); return FALSE;}
-
+	if(m_posCur==m_fileList.end()/*(FileList::iterator)NULL*/) {ASSERT(0); return FALSE;}
 	BOOL	ret=FALSE;
 
 	SExtractDest	dest;
@@ -690,13 +709,13 @@ BOOL CUnAlz::ExtractCurrentFile(const char* szDestPathName, const char* szDestFi
 	}
 	
 	// θ
-	strcpy(szDestPathFileName, szDestPathName);
+	safe_strcpy(szDestPathFileName, szDestPathName, MAX_PATH);
 	if(szDestPathFileName[strlen(szDestPathFileName)]!=PATHSEPC)
-		strcat(szDestPathFileName, PATHSEP);
+		safe_strcat(szDestPathFileName, PATHSEP, MAX_PATH);
 
 	// ϸ
-	if(szDestFileName) strcat(szDestPathFileName, szDestFileName);
-	else strcat(szDestPathFileName, m_posCur->fileName);
+	if(szDestFileName) safe_strcat(szDestPathFileName, szDestFileName, MAX_PATH);
+	else safe_strcat(szDestPathFileName, m_posCur->fileName, MAX_PATH);
 
 	// ../../    Ȯ
 	if( strstr(szDestPathFileName, "../")||
@@ -707,8 +726,6 @@ BOOL CUnAlz::ExtractCurrentFile(const char* szDestPathName, const char* szDestFi
 		return FALSE;
 	}
 
-
-
 #ifndef _WIN32 
 	{
 		char* p = szDestPathFileName;			//  delimiter ٲٱ 
@@ -820,7 +837,7 @@ BOOL CUnAlz::ExtractTo(SExtractDest* dest)
 /// @date    2004-03-06  11:09:17
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 /*
-BOOL CUnAlz::ExtractDeflate(FILE* fp, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractDeflate(FILE* fp, SAlzLocalFileHeader& file)
 {
 	z_stream	stream;
 	BYTE*		pInBuf=NULL;
@@ -938,13 +955,15 @@ BOOL CUnAlz::DigPath(const char* szPathName)
 		if(strlen(path)==0)
 		{
 			if(szPathName[0]=='/')			// is absolute path ?
-				strcpy(path,"/");
-			strcat(path, token);
+				safe_strcpy(path,"/", MAX_PATH);
+			else if(szPathName[0]=='\\' && szPathName[1]=='\\')	// network drive ?
+				safe_strcpy(path,"\\\\", MAX_PATH);
+			safe_strcat(path, token, MAX_PATH);
 		}
 		else
 		{
-			strcat(path, PATHSEP);
-			strcat(path, token);
+			safe_strcat(path, PATHSEP,MAX_PATH);
+			safe_strcat(path, token,MAX_PATH);
 		}
 
 		if(IsFolder(path)==FALSE)
@@ -1030,7 +1049,7 @@ int	CUnAlz::WriteToDest(SExtractDest* dest, BYTE* buf, int nSize)
 #define BZIP2_TAIL_SIZE		10		//  4+5 .?
 BYTE bzip2Header[BZIP2_HEADER_SIZE] = {0x42, 0x5a, 0x68, 0x39, 0x31, 0x41, 0x59, 0x26, 0x53, 0x59};
 
-BOOL CUnAlz::ExtractBzip2_bak(FILE* fp, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractBzip2_bak(FILE* fp, SAlzLocalFileHeader& file)
 {
 	bz_stream	stream;
 	BYTE*		pInBuf=NULL;
@@ -1123,7 +1142,7 @@ END :
 /// @date    2004-03-06  11:10:53
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 #define BUF_LEN		(4096*2)
-BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
 	BOOL		ret = FALSE;
 	BYTE		buf[BUF_LEN];
@@ -1199,7 +1218,7 @@ BOOL CUnAlz::ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file)
 /// @date    2004-03-01  5:47:36
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 #define BZIP2_EXTRACT_BUF_SIZE	0x2000
-BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
 	BZFILE		*bzfp = NULL;
 	int			smallMode = 0;
@@ -1318,7 +1337,7 @@ BOOL CUnAlz::ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file)
 /// @return  
 /// @date    2004-03-06  11:11:36
 ////////////////////////////////////////////////////////////////////////////////////////////////////
-BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file)
+BOOL CUnAlz::ExtractDeflate2(SExtractDest* dest, SAlzLocalFileHeader& file)
 {
 	z_stream	stream;
 	BYTE		pInBuf[IN_BUF_SIZE];
@@ -1464,7 +1483,9 @@ BOOL CUnAlz::FOpen(const char* szPathName)
 	m_bIsEOF = FALSE;
 	for(i=0;i<MAX_FILES;i++)						// aa.alz ϸ  aa.a00 aa.a01 aa.a02 .. 
 	{
-		if(i>0) sprintf(temp+nLen-3, "%c%02d", (i-1)/100+'a', (i-1)%100);
+		if(i>0) 
+			safe_sprintf(temp+nLen-3, 4, "%c%02d", (i-1)/100+'a', (i-1)%100);
+
 #ifdef _WIN32
 		m_files[i].fp = CreateFileA(temp, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
 		if(m_files[i].fp==INVALID_HANDLE_VALUE) break;
@@ -1689,7 +1710,6 @@ BOOL	CUnAlz::chkValidPassword()
 
 
 /*
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 //	from CZipArchive
 //	Copyright (C) 2000 - 2004 Tadeusz Dracz
@@ -1727,7 +1747,7 @@ void CUnAlz::CryptUpdateKeys(CHAR c)
 BOOL CUnAlz::CryptCheck(CHAR *buf)
 {
 	CHAR b = 0;
-	for (int i = 0; i < ENCR_HEADER_LEN; i++)
+	for (int i = 0; i < ALZ_ENCR_HEADER_LEN; i++)
 	{
 		b = buf[i]; 
 		CryptDecode((CHAR&)b);
@@ -1756,7 +1776,6 @@ UINT32 CUnAlz::CryptCRC32(UINT32 l, CHAR c)
 	const ULONG *CRC_TABLE = get_crc_table();
 	return CRC_TABLE[(l ^ c) & 0xff] ^ (l >> 8);
 }
-
 */
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -1817,11 +1836,11 @@ BOOL CUnAlz::CryptCheck(const BYTE* buf)
 {
 	int i;
 	BYTE c;
-	BYTE temp[ENCR_HEADER_LEN];
+	BYTE temp[ALZ_ENCR_HEADER_LEN];
 
-	memcpy(temp, buf, ENCR_HEADER_LEN);		// ӽ .
+	memcpy(temp, buf, ALZ_ENCR_HEADER_LEN);		// ӽ .
 
-	for(i=0;i<ENCR_HEADER_LEN;i++)
+	for(i=0;i<ALZ_ENCR_HEADER_LEN;i++)
 	{
 		c = temp[i] ^ DecryptByte();
 		UpdateKeys(c);
@@ -1877,8 +1896,68 @@ void CUnAlz::DecryptingData(int nSize, BYTE* data)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 UINT32 CUnAlz::CRC32(UINT32 l, BYTE c)
 {
-	const ULONG *CRC_TABLE = get_crc_table();
+	const unsigned long *CRC_TABLE = get_crc_table();
 	return CRC_TABLE[(l ^ c) & 0xff] ^ (l >> 8);
 }
 
+void CUnAlz::SetPassword(char *passwd) 
+{ 
+	if(strlen(passwd) == 0) return; 
+	safe_strcpy(m_szPasswd, passwd, UNALZ_LEN_PASSWORD); 
+}
+
+#ifdef _UNALZ_ICONV
+void CUnAlz::SetDestCodepage(const char* szToCodepage)
+{
+	safe_strcpy(m_szToCodepage, szToCodepage, UNALZ_LEN_CODEPAGE); 
+}
+#endif
+
+
+
+////////////////////////////////////////////////////////////////////////////////////////////////////
+///          ڿ ó Լ
+/// @param   l  
+/// @param   c  
+/// @return  
+/// @date    2007-02
+////////////////////////////////////////////////////////////////////////////////////////////////////
+unsigned int CUnAlz::_strlcpy (char *dest, const char *src, unsigned int size)
+{
+	register unsigned int i = 0;
+	if (size > 0) {
+	size--;
+	for (i=0; size > 0 && src[i] != '\0'; ++i, size--)
+		dest[i] = src[i];
+	dest[i] = '\0';
+	}
+	while (src[i++]);
+	return i;
+}
+unsigned int CUnAlz::_strlcat (char *dest, const char *src, unsigned int size)
+{
+	register char *d = dest;
+	for (; size > 0 && *d != '\0'; size--, d++);
+	return (d - dest) + _strlcpy(d, src, size);
+}
+
+//  strcpy
+void CUnAlz::safe_strcpy(char* dst, const char* src, size_t dst_size)
+{
+#ifdef _WIN32
+	lstrcpynA(dst, src, dst_size);
+#else
+	_strlcpy(dst, src, dst_size);
+#endif
+}
+
+void CUnAlz::safe_strcat(char* dst, const char* src, size_t dst_size)
+{
+#ifdef _WIN32
+	StringCchCatExA(dst, dst_size, src, NULL, NULL, STRSAFE_FILL_BEHIND_NULL);
+	//lstrcatA(dst, src);			// not safe!!
+#else
+	_strlcat(dst, src, dst_size);
+#endif
+}
 
