@@ -1,21 +1,50 @@
 /*
+  UNALZ : read and extract module for ALZ format.
 
-  COPYRIGHT(C) 2004-2006  hardkoder@gmail , http://www.kipple.pe.kr
+  LICENSE (zlib License)
+  Copyright (C) 2004-2007 kippler@gmail.com , http://www.kipple.pe.kr
 
-  ۱  : ( BSD License )
-    -  ҽ  // մϴ.
-    - ,  ٰ ϰų ϸ ȵ.
+  This software is provided 'as-is', without any express or implied
+  warranty.  In no event will the authors be held liable for any damages
+  arising from the use of this software.
+
+  Permission is granted to anyone to use this software for any purpose,
+  including commercial applications, and to alter it and redistribute it
+  freely, subject to the following restrictions:
+
+  1. The origin of this software must not be misrepresented; you must not
+     claim that you wrote the original software. If you use this software
+     in a product, an acknowledgment in the product documentation would be
+     appreciated but is not required.
+  2. Altered source versions must be plainly marked as such, and must not be
+     misrepresented as being the original software.
+  3. This notice may not be removed or altered from any source distribution.
+
+   Ʈ   Ǵ    "ִ ״" ˴ϴ.  
+   쿡 ۼڴ  Ʈ   ؿ  å  ʽϴ.
+
+     ؼϴ 쿡 Ͽ   α׷ ϴ  뵵  Ʈ 
+  ϰ Ӱ     ִ  Գ ο˴ϴ.
+
+  1.  Ʈ ó ߸ ǥϰų  Ʈ ڽ ۼߴٰ ؼ  ˴ϴ. ǰ 
+      Ʈ ϴ  䱸  ƴ ǰ    ־ ֽø ϰڽϴ.
+  2.  ҽ  ݵ Ȯϰ ǥõǾ ϸ  Ʈ εǵ ߸ ǥؼ  ˴ϴ.
+  3.  ҽ     ϰų   ϴ.
+
+  =============================================================================================================
+
+   :
     - ҽ  ãҰų,  Ͽ  ̿   ˷ָ  ﲬ..
-	- ڽ α׷  ҽ Ͽ   ڿ ˷ָ  ڰ  Ҳ..
+	- ڽ α׷  ҽ Ͽ    ָ   Ҳ..
 
   ҽ  : 
 	- .ALZ   ϱ  . 
 	- ALZ BZIP2(̶  Ѱ ƴϰ,  CRC   ũ⸦  ), 
 	  DEFLATE  ˰ ZIP     .
-	  ( bzip2  4.9x  Ͽ, deflate  5.x  Ͽ. 5.x ʹ bzip2 ʹ    )
+	  (bzip2  4.9x  Ͽ, deflate  5.x  Ͽ. 5.x ʹ bzip2 ʹ   )
 	- UnAlzBz2decompress.c  UnAlzbzlib.c   bzip2 ҽ alz   
-	  ( deflate   ȵǾ  ׳ zlib  ᵵ .. bzip2   Ǿ 
-	   bzip2 ҽ ״  ȵȴ. )
+	  (deflate   ȵǾ  ׳ zlib  ᵵ .. bzip2   Ǿ 
+	   bzip2 ҽ ״  ȵȴ.)
 	-  ҽ 4ĭ   Ͽ.
 
     :
@@ -28,11 +57,11 @@
 	2004/02/08	- bzip2  
 	2004/03/01	- bzip2  Ϻ  Ҭ
 				- callback ..
-	2004/03/07	- ƿ Լ ߰ ( ExtractCurrentFileToBuf() )
-	2004/10/03	-     ߰ ( FILE I/O    Ŭ  )
+	2004/03/07	- ƿ Լ ߰ (ExtractCurrentFileToBuf())
+	2004/10/03	-     ߰ (FILE I/O    Ŭ )
 	            - 2GB ̻  ó  (WINDOWS ONLY)
 	2004/10/22	-  ÷(BSD/LINUX)  
-				  ( BSD/LINUX   2GB  ϸ  )
+				  (BSD/LINUX   2GB  ϸ )
 				- unalz 0.20
 	2004/10/23	- by xxfree86 : DARWIN  , θ "\\" Խ  
 	2004/10/24	- by aqua0125 : ڵ ȯó, 64bit  ó
@@ -49,7 +78,7 @@
 				- ȣɸ ϰ Ȱɸ   ó
 				-  ޺κ ߷ ջ ϵ  κб  Ǯ 
 				- unalz 0.31
-	2005/01/08	- ȣ ߸ Է½ ѹ üũ ι°  ȣ Էؼ Ǯ ϰ Ǵ  
+	2005/01/08	- ȣ ߸ Է½ ѹ üũ ι°  ȣ Էص Ǯ ϰ Ǵ  
 	2005/02/05	-   deflate   CRC Ȯ  ߰
 	2005/03/07	- bzip2, raw Ͽ   CRC Ȯ  ߰
 	2005/03/13	- ALZ  ƴҰ  ڵ(ERR_NOT_ALZ_FILE) ߰
@@ -70,11 +99,20 @@
 				- unalz 0.51
 	2005/07/27	- main()  setlocale() ߰
 				- unalz 0.52
-	2005/10/15	- NetBSD   ǵ  ( by minskim@bawi )
-	2005/11/21	- buffer overflow   ( by Ulf Harnhammar )
+	2005/10/15	- NetBSD   ǵ  (by minskim@bawi)
+	2005/11/21	- buffer overflow   (by Ulf Harnhammar)
 				- unalz 0.53
-	2006/03/10	- ..      (by vuln@secunia )
+	2006/03/10	- ..      (by vuln@secunia)
 				- unalz 0.55
+	2006/04/23	-  ó Ÿӿ ϵ 
+	2006/12/31	- strcpy/strcat/sprintf    ÷ο ɼ ִ Լ  (by liam.joo@gmail)
+				- unalz 0.60
+	2007/02/10	-  strlcpy, strlcat   
+				- unalz 0.61
+	2007/04/12	- unalz command   ȣ Է½ \n ԵǴ  
+	2008/04/04	- debian    (by cwryu@debian )
+				- ҽ , NULL iterator  
+				- unalz 0.62
   
    :
 	- alz    (deflate/ bzip2/raw)
@@ -85,7 +123,7 @@
 	- CRC üũ
 
 
-   ɼ ( -DXXXX )
+   ɼ (-DXXXX)
 	- _WIN32 : WIN32 
 	- _UNALZ_ICONV : iconv  ؼ code  ȯ 
 	- _UNALZ_UTF8 : _UNALZ_ICONV    ⺻ ڵ "UTF-8"  
@@ -96,6 +134,8 @@
 #ifndef _UNALZ_H_
 #define _UNALZ_H_
 
+#include <cstdlib>
+#include <cstring>
 #include <vector>
 using namespace std;
 
@@ -142,9 +182,6 @@ using namespace std;
 #ifndef LONG
 	typedef long LONG;
 #endif
-#ifndef ULONG
-	typedef unsigned long ULONG;   // same as DWORD? i don't know.
-#endif
 #ifndef BOOL
 #	ifndef BOOL_DEFINED		// ̹ BOOL  DEFINE Ǿ  BOOL_DEFINED  define ؼ     ִ.
 	typedef int BOOL;
@@ -181,10 +218,10 @@ namespace UNALZ
 #	pragma pack(1)
 #endif
 
-static const char UNALZ_VERSION[]   = "CUnAlz0.55";
-static const char UNALZ_COPYRIGHT[] = "Copyright(C) 2004-2006 by hardkoder@gmail ( http://www.kipple.pe.kr ) ";
+static const char UNALZ_VERSION[]   = "CUnAlz0.62";
+static const char UNALZ_COPYRIGHT[] = "Copyright(C) 2004-2008 by kippler@gmail.com ( http://www.kipple.pe.kr ) ";
 
-enum		{ENCR_HEADER_LEN=12}; // xf86
+enum		{ALZ_ENCR_HEADER_LEN=12}; // xf86
 //   ..
 struct SAlzHeader
 {
@@ -232,7 +269,7 @@ enum ALZ_FILE_DESCRIPTOR
 	ALZ_FILE_DESCRIPTOR_FILESIZEFIELD_8BYTE = 0x80,
 };
 
-struct _SLocalFileHeaderHead			///<   .
+struct _SAlzLocalFileHeaderHead			///<   .
 {
 	SHORT	fileNameLength;
 	BYTE    fileAttribute;			    // from http://www.zap.pe.kr, enum FILE_ATTRIBUE 
@@ -256,21 +293,12 @@ struct _SLocalFileHeaderHead			///<   .
 	*/
 };
 
-/*
-struct _SDataDescriptor
+struct SAlzLocalFileHeader
 {
-	UINT32	crc32;
-	UINT32	compressed;
-	UINT32	uncompressed;
-};
-*/
-
-struct SLocalFileHeader
-{
-	SLocalFileHeader() { memset(this, 0, sizeof(*this)); }
-	//~SLocalFileHeader() { if(fileName) free(fileName); if(extraField) free(extraField); }
+	SAlzLocalFileHeader() { memset(this, 0, sizeof(*this)); }
+	//~SAlzLocalFileHeader() { if(fileName) free(fileName); if(extraField) free(extraField); }
 	void Clear() { if(fileName) free(fileName); fileName=NULL; if(extraField) free(extraField);extraField=NULL; }
-	_SLocalFileHeaderHead	head;
+	_SAlzLocalFileHeaderHead	head;
 
 	BYTE					compressionMethod;			///<   : 2 - deflate, 1 -  bzip2, 0 -  .
 	BYTE					unknown;
@@ -281,13 +309,12 @@ struct SLocalFileHeader
 
 	CHAR*					fileName;
 	BYTE*					extraField;
-//	_SDataDescriptor		dataDescriptor;
 	INT64					dwFileDataPos;				///<  file data   ġ..
 	
-	BYTE					encChk[ENCR_HEADER_LEN];	// xf86
+	BYTE					encChk[ALZ_ENCR_HEADER_LEN];	// xf86
 };
 
-struct _SCentralDirectoryStructureHead
+struct _SAlzCentralDirectoryStructureHead
 {
 	UINT32	dwUnknown;						///<  ׻ NULL ̴..
 	UINT32	dwUnknown2;						///<  Ƹ crc
@@ -316,7 +343,7 @@ struct SCentralDirectoryStructure
 {
 	SCentralDirectoryStructure() { memset(this, 0, sizeof(*this)); }
 	//~SCentralDirectoryStructure() { if(fileName) free(fileName); if(extraField) free(extraField);if(fileComment)free(fileComment); }
-	_SCentralDirectoryStructureHead	head;
+	_SAlzCentralDirectoryStructureHead	head;
 	/*
 	CHAR*	fileName;
 	BYTE*	extraField;
@@ -351,16 +378,13 @@ struct SEndOfCentralDirectoryRecord
 #ifdef _WIN32
 #	pragma pack(pop, UNALZ)		///<  PACKING  
 #else
-#	ifdef __LP64__				// 64bit  ŷ 8byte ³ ? ߸𸣰ڴ.....
-#		pragma pack(8)
-#	else
-#		pragma pack(4)
-#	endif
+#	pragma pack()				// restore packing
 #endif
 
 
+
 ///<  PROGRESS CALLBACK FUNCTION -    Ȳ ˰  ̰  ȴ.
-typedef void (_UnAlzCallback)(const char* szMessage, INT64 nCurrent, INT64 nRange, void* param, BOOL* bHalt);
+typedef void (_UnAlzCallback)(const char* szFileName, INT64 nCurrent, INT64 nRange, void* param, BOOL* bHalt);
 
 
 class CUnAlz  
@@ -377,12 +401,12 @@ public:
 	void	SetCallback(_UnAlzCallback* pFunc, void* param=NULL);
 	void	SetPipeMode(BOOL bPipeMode) {m_bPipeMode=bPipeMode;}
 
-	void	setPassword(char *passwd) { if(strlen(passwd) == 0) return; strcpy(m_szPasswd, passwd); };  // xf86
+	void	SetPassword(char *passwd);  // xf86
 	BOOL	chkValidPassword();			// xf86
 	BOOL	IsEncrypted() { return m_bIsEncrypted; };
 
 #ifdef _UNALZ_ICONV
-	void	SetDestCodepage(const char* szToCodepage) { strcpy(m_szToCodepage, szToCodepage); }
+	void	SetDestCodepage(const char* szToCodepage);
 #endif
 
 public :			///<  WIN32  ( UNICODE ó )
@@ -399,7 +423,7 @@ public :			///<  WIN32  ( UNICODE ó )
 #endif // _WIN32
 
 public :
-	typedef vector<SLocalFileHeader>		FileList;					///<   .
+	typedef vector<SAlzLocalFileHeader>		FileList;					///<   .
 	FileList*			GetFileList() { return &m_fileList; };			///<  file  
 	void				SetCurrentFile(FileList::iterator newPos);		///< low level ..
 	FileList::iterator	GetCurFileHeader() { return m_posCur; };		///<   (SetCurrentFile()  õ)  
@@ -464,6 +488,12 @@ public :
 	static const char*	GetCopyright() { return UNALZ_COPYRIGHT; }
 	BOOL				IsHalted() { return m_bHalt; }		// by xf86
 
+public :
+	static void			safe_strcpy(char* dst, const char* src, size_t dst_size);
+	static void			safe_strcat(char* dst, const char* src, size_t dst_size);
+	static unsigned int _strlcpy (char *dest, const char *src, unsigned int size);
+	static unsigned int _strlcat (char *dest, const char *src, unsigned int size);
+
 private :
 	SIGNATURE	ReadSignature();
 	BOOL		ReadAlzFileHeader();
@@ -491,11 +521,11 @@ private :
 private :
 	BOOL		ExtractTo(SExtractDest* dest);
 
-	//BOOL		ExtractDeflate(FILE* fp, SLocalFileHeader& file);
-	//BOOL		ExtractBzip2_bak(FILE* fp, SLocalFileHeader& file); - (߸)  
-	BOOL		ExtractDeflate2(SExtractDest* dest, SLocalFileHeader& file);
-	BOOL		ExtractBzip2(SExtractDest* dest, SLocalFileHeader& file);
-	BOOL		ExtractRawfile(SExtractDest* dest, SLocalFileHeader& file);
+	//BOOL		ExtractDeflate(FILE* fp, SAlzLocalFileHeader& file);
+	//BOOL		ExtractBzip2_bak(FILE* fp, SAlzLocalFileHeader& file); - (߸)  
+	BOOL		ExtractDeflate2(SExtractDest* dest, SAlzLocalFileHeader& file);
+	BOOL		ExtractBzip2(SExtractDest* dest, SAlzLocalFileHeader& file);
+	BOOL		ExtractRawfile(SExtractDest* dest, SAlzLocalFileHeader& file);
 
 private :		// bzip2  ó Լ..
 	typedef void MYBZFILE;
@@ -534,7 +564,8 @@ private :		//    ó  (lapper^^?) Ŭ
 
 	BOOL		m_bIsEncrypted;		// xf86
 	BOOL		m_bIsDataDescr;
-	char		m_szPasswd[512];
+#define UNALZ_LEN_PASSWORD	512
+	char		m_szPasswd[UNALZ_LEN_PASSWORD];
 	BOOL		m_bPipeMode;						///< pipemode - ޽ ¾ stdout θ 
 
 private :
@@ -568,8 +599,10 @@ private :
 	BOOL				m_bHalt;
 
 #ifdef _UNALZ_ICONV
-	char				m_szToCodepage[256];		///< codepage 
-	char				m_szFromCodepage[256];		///< "CP949"
+
+#define UNALZ_LEN_CODEPAGE	256
+	char				m_szToCodepage[UNALZ_LEN_CODEPAGE];		///< codepage 
+	char				m_szFromCodepage[UNALZ_LEN_CODEPAGE];		///< "CP949"
 #endif
 };
 }
